/*
  Copyright 2019 Kakao Corp.

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
@file:JvmName("ServerHostsKt")

package com.kakao.sdk.partner.model

import com.kakao.sdk.common.KakaoSdk
import com.kakao.sdk.common.model.ServerHosts

internal class PhasedServerHosts(val phase: KakaoPhase) : ServerHosts() {
    override val kauth: String =
        when (phase) {
            KakaoPhase.DEV -> "alpha-kauth.kakao.com"
            KakaoPhase.SANDBOX -> "sandbox-kauth.kakao.com"
            KakaoPhase.CBT -> "beta-kauth.kakao.com"
            KakaoPhase.PRODUCTION -> "kauth.kakao.com"
        }

    override val kapi: String =
        when (phase) {
            KakaoPhase.DEV -> "alpha-kapi.kakao.com"
            KakaoPhase.SANDBOX -> "sandbox-kapi.kakao.com"
            KakaoPhase.CBT -> "beta-kapi.kakao.com"
            KakaoPhase.PRODUCTION -> "kapi.kakao.com"
        }

    override val account: String =
        when (phase) {
            KakaoPhase.DEV -> "alpha-accounts.kakao.com"
            KakaoPhase.SANDBOX -> "sandbox-accounts.kakao.com"
            KakaoPhase.CBT -> "beta-accounts.kakao.com"
            KakaoPhase.PRODUCTION -> "accounts.kakao.com"
        }

    override val mobileAccount: String =
        when (phase) {
            KakaoPhase.DEV -> "alpha-auth.kakao.com"
            KakaoPhase.SANDBOX -> "sandbox-auth.kakao.com"
            KakaoPhase.CBT -> "beta-auth.kakao.com"
            KakaoPhase.PRODUCTION -> "auth.kakao.com"
        }

    override val sharer: String =
        when (phase) {
            KakaoPhase.DEV -> "alpha-sharer.devel.kakao.com"
            KakaoPhase.SANDBOX -> "sandbox-sharer.devel.kakao.com"
            KakaoPhase.CBT -> "beta-sharer.kakao.com"
            KakaoPhase.PRODUCTION -> "sharer.kakao.com"
        }

    override val navi: String =
        when (phase) {
            KakaoPhase.DEV, KakaoPhase.SANDBOX -> "sandbox-kakaonavi-wguide.kakao.com"
            KakaoPhase.CBT, KakaoPhase.PRODUCTION -> "kakaonavi-wguide.kakao.com"
        }

    override val channel: String =
        when (phase) {
            KakaoPhase.DEV, KakaoPhase.SANDBOX -> "sandbox-pf.kakao.com"
            KakaoPhase.CBT -> "beta-pf.kakao.com"
            KakaoPhase.PRODUCTION -> "pf.kakao.com"
        }
}

/**
 * [KakaoPhase] 를 기반으로 원하는 Phase의 서버 호스트 정보를 생성. [KakaoSdk.init] 파라미터로 사용.
 *
 *  ```kotlin
 *  // Sandbox 설정 예제
 *  KakaoSdk.init(
 *    context,
 *    appKey,
 *    ServerHosts.withPhase(KakaoPhase.SANDBOX)
 *  )
 *  ```
 */
fun ServerHosts.Companion.withPhase(phase: KakaoPhase): ServerHosts = PhasedServerHosts(phase)