/*
  Copyright 2021 Kakao Corp.

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
package com.kakao.sdk.partner.friend

import android.content.Context
import com.kakao.sdk.partner.friend.client.FriendPickerClient
import com.kakao.sdk.partner.friend.model.MultiPickerRequestParams
import com.kakao.sdk.partner.friend.model.PickerRequestParams
import com.kakao.sdk.partner.friend.model.SelectedUsers
import io.reactivex.Single


class RxFriendPickerManager {

    internal fun startActivity(context: Context, param: PickerRequestParams) : Single<SelectedUsers> =
        Single.create<SelectedUsers> { emitter ->

            val callback : (SelectedUsers?, Throwable?) -> Unit = { users, error ->
                if (error != null) {
                    emitter.onError(error)
                } else {
                    users?.let {
                        emitter.onSuccess(users)
                    }
                }
            }

            if (param is MultiPickerRequestParams) {
                FriendPickerClient.instance.selectMultiple(context, param, callback)
            }
            else {
                FriendPickerClient.instance.selectSingle(context, param, callback)
            }
        }
}
