/*
  Copyright 2021 Kakao Corp.

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
package com.kakao.sdk.partner.friend

import com.kakao.sdk.auth.network.kapiWithOAuth
import com.kakao.sdk.network.ApiCallback
import com.kakao.sdk.network.ApiFactory
import com.kakao.sdk.partner.friend.model.PickerFriendFilter
import com.kakao.sdk.partner.friend.model.PickerFriendOrder
import com.kakao.sdk.partner.friend.model.PickerFriendType
import com.kakao.sdk.partner.friend.network.FriendApi
import com.kakao.sdk.partner.friend.network.model.Me
import com.kakao.sdk.partner.friend.network.model.PickerFriend
import com.kakao.sdk.partner.friend.network.model.PickerFriends

/**
 * @suppress
 */
internal object Repository {
    private val friendApi: FriendApi = ApiFactory.kapiWithOAuth.create(FriendApi::class.java)
    var friends = mutableListOf<PickerFriend>()
    var me: Me? = null

    fun loadFriendsData(
        friendOrder: PickerFriendOrder?,
        friendType: PickerFriendType,
        friendFilter: PickerFriendFilter,
        countryCodes: List<String>?,
        callback: (error: Throwable?) -> Unit
    ) {
        pickerFriendsForPartner(
            friendOrder,
            friendType,
            friendFilter,
            countryCodes
        ) { data, error ->
            if (error != null) {
                callback(error)
            } else if (data != null) {
                data.friends.elements?.let {
                    friends.addAll(it)
                    me = data.me
                    callback(null)
                }
            }
        }
    }

    private fun pickerFriendsForPartner(
        friendOrder: PickerFriendOrder?,
        friendType: PickerFriendType,
        friendFilter: PickerFriendFilter,
        countryCodes: List<String>?,
        callback: (friends: PickerFriends<PickerFriend>?, error: Throwable?) -> Unit
    ) {
        friendApi.friends(
            friendOrder = friendOrder,
            friendType = friendType,
            friendFilter = friendFilter,
            limit = Constants.PARTNER_FRIENDS_API_LIMIT,
            countryCodes = countryCodes?.joinToString(",")
        ).enqueue(object : ApiCallback<PickerFriends<PickerFriend>>() {
            override fun onComplete(model: PickerFriends<PickerFriend>?, error: Throwable?) {
                callback(model, error)
            }
        })
    }

    fun clearData() {
        me = null
        friends.clear()
    }
}